/*******************************************************************************
 * Copyright (c) 2000, 2023 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *    IBM Corporation - initial API and implementation
 *******************************************************************************/

/* Note: This file was auto-generated by org.eclipse.swt.tools.internal.JNIGenerator */
/* DO NOT EDIT - your changes will be lost. */

#ifndef COM_NATIVE_ENTER
#define COM_NATIVE_ENTER(env, that, func) 
#endif
#ifndef COM_NATIVE_EXIT
#define COM_NATIVE_EXIT(env, that, func) 
#endif

typedef enum {
	CAUUID_1sizeof_FUNC,
	CLSIDFromProgID_FUNC,
	CLSIDFromString_FUNC,
	CONTROLINFO_1sizeof_FUNC,
	CoCreateInstance_FUNC,
	CoFreeUnusedLibraries_FUNC,
	CoGetClassObject_FUNC,
	CoLockObjectExternal_FUNC,
	CreateCoreWebView2EnvironmentWithOptions_FUNC,
	CreateStdAccessibleObject_FUNC,
	CreateSwtWebView2Callback_FUNC,
	CreateSwtWebView2Host_FUNC,
	CreateSwtWebView2Options_FUNC,
	DISPPARAMS_1sizeof_FUNC,
	DoDragDrop_FUNC,
	ELEMDESC_1sizeof_FUNC,
	EXCEPINFO_1sizeof_FUNC,
	FORMATETC_1sizeof_FUNC,
	FUNCDESC_1sizeof_FUNC,
	GUID_1sizeof_FUNC,
	GetClassFile_FUNC,
	IIDFromString_FUNC,
	IsEqualGUID_FUNC,
	LICINFO_1sizeof_FUNC,
	LresultFromObject_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_ole_win32_FORMATETC_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_ole_win32_OLEINPLACEFRAMEINFO_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_ole_win32_STGMEDIUM_2I_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_ole_win32_DISPPARAMS_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_ole_win32_FORMATETC_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_ole_win32_FUNCDESC_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_ole_win32_GUID_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_ole_win32_STGMEDIUM_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_ole_win32_TYPEATTR_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_ole_win32_VARDESC_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_ole_win32_VARIANT_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_RECT_2JI_FUNC,
	OLECMD_1sizeof_FUNC,
	OLEINPLACEFRAMEINFO_1sizeof_FUNC,
	OleCreate_FUNC,
	OleCreateFromFile_FUNC,
	OleCreatePropertyFrame_FUNC,
	OleDraw_FUNC,
	OleFlushClipboard_FUNC,
	OleGetClipboard_FUNC,
	OleIsCurrentClipboard_FUNC,
	OleIsRunning_FUNC,
	OleRun_FUNC,
	OleSave_FUNC,
	OleSetClipboard_FUNC,
	OleSetContainedObject_FUNC,
	OleSetMenuDescriptor_FUNC,
	OleTranslateColor_FUNC,
	PathToPIDL_FUNC,
	ProgIDFromCLSID_FUNC,
	RegisterDragDrop_FUNC,
	ReleaseStgMedium_FUNC,
	RevokeDragDrop_FUNC,
	SHCreateItemFromParsingName_FUNC,
	SHCreateMemStream_FUNC,
	STGMEDIUM_1sizeof_FUNC,
	StgCreateDocfile_FUNC,
	StgIsStorageFile_FUNC,
	StgOpenStorage_FUNC,
	SysAllocString_FUNC,
	SysAllocStringLen_FUNC,
	SysFreeString_FUNC,
	SysStringByteLen_FUNC,
	SysStringLen_FUNC,
	TYPEATTR_1sizeof_FUNC,
	TYPEDESC_1sizeof_FUNC,
	VARDESC_1sizeof_FUNC,
	VARIANT_1sizeof_FUNC,
	VariantChangeType_FUNC,
	VariantClear_FUNC,
	VariantInit_FUNC,
	VtblCall__IJ_FUNC,
	VtblCall__IJD_FUNC,
	VtblCall__IJI_FUNC,
	VtblCall__IJIIILorg_eclipse_swt_internal_ole_win32_DISPPARAMS_2JLorg_eclipse_swt_internal_ole_win32_EXCEPINFO_2J_FUNC,
	VtblCall__IJIIJLorg_eclipse_swt_internal_win32_SIZE_2_FUNC,
	VtblCall__IJIJ_FUNC,
	VtblCall__IJIJI_3J_FUNC,
	VtblCall__IJIJJ_FUNC,
	VtblCall__IJIJ_3I_FUNC,
	VtblCall__IJILorg_eclipse_swt_internal_ole_win32_GUID_2_FUNC,
	VtblCall__IJILorg_eclipse_swt_internal_ole_win32_GUID_2IILorg_eclipse_swt_internal_ole_win32_DISPPARAMS_2JLorg_eclipse_swt_internal_ole_win32_EXCEPINFO_2_3I_FUNC,
	VtblCall__IJILorg_eclipse_swt_internal_ole_win32_GUID_2JJ_FUNC,
	VtblCall__IJILorg_eclipse_swt_internal_ole_win32_GUID_2Lorg_eclipse_swt_internal_ole_win32_GUID_2Lorg_eclipse_swt_internal_ole_win32_GUID_2_FUNC,
	VtblCall__IJILorg_eclipse_swt_internal_win32_MSG_2JIJLorg_eclipse_swt_internal_win32_RECT_2_FUNC,
	VtblCall__IJILorg_eclipse_swt_internal_win32_SIZE_2_FUNC,
	VtblCall__IJI_3I_FUNC,
	VtblCall__IJI_3J_FUNC,
	VtblCall__IJI_3JI_3I_FUNC,
	VtblCall__IJI_3J_3I_FUNC,
	VtblCall__IJI_3J_3J_3I_3J_FUNC,
	VtblCall__IJJ_FUNC,
	VtblCall__IJJI_FUNC,
	VtblCall__IJJII_3J_FUNC,
	VtblCall__IJJI_3I_FUNC,
	VtblCall__IJJI_3J_FUNC,
	VtblCall__IJJJ_FUNC,
	VtblCall__IJJJI_3J_FUNC,
	VtblCall__IJJJJ_FUNC,
	VtblCall__IJJJJJJ_FUNC,
	VtblCall__IJJJLorg_eclipse_swt_internal_ole_win32_GUID_2J_3J_FUNC,
	VtblCall__IJJJLorg_eclipse_swt_internal_win32_POINT_2I_FUNC,
	VtblCall__IJJJ_3J_FUNC,
	VtblCall__IJJLorg_eclipse_swt_internal_win32_POINT_2J_FUNC,
	VtblCall__IJJ_3I_FUNC,
	VtblCall__IJJ_3J_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_CAUUID_2_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_CONTROLINFO_2_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_FORMATETC_2_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_FORMATETC_2Lorg_eclipse_swt_internal_ole_win32_STGMEDIUM_2_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_GUID_2_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_GUID_2IIJJ_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_GUID_2ILorg_eclipse_swt_internal_ole_win32_OLECMD_2J_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_GUID_2JII_3I_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_GUID_2Lorg_eclipse_swt_internal_ole_win32_GUID_2_3J_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_GUID_2_3J_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_ole_win32_LICINFO_2_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_win32_MSG_2_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_win32_POINT_2I_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_win32_PROPERTYKEY_2J_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_win32_RECT_2_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_win32_RECT_2JI_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_win32_RECT_2JJ_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_win32_RECT_2Lorg_eclipse_swt_internal_win32_RECT_2_FUNC,
	VtblCall__IJLorg_eclipse_swt_internal_win32_TF_1DISPLAYATTRIBUTE_2_FUNC,
	VtblCall__IJ_3C_FUNC,
	VtblCall__IJ_3CI_FUNC,
	VtblCall__IJ_3CIII_3J_FUNC,
	VtblCall__IJ_3CII_3I_3I_FUNC,
	VtblCall__IJ_3CJ_FUNC,
	VtblCall__IJ_3CJIII_3J_FUNC,
	VtblCall__IJ_3CJII_3J_FUNC,
	VtblCall__IJ_3C_3C_FUNC,
	VtblCall__IJ_3C_3CJ_3C_3J_FUNC,
	VtblCall__IJ_3C_3C_3C_3C_3J_FUNC,
	VtblCall__IJ_3C_3J_FUNC,
	VtblCall__IJ_3I_FUNC,
	VtblCall__IJ_3ILorg_eclipse_swt_internal_ole_win32_GUID_2_3J_FUNC,
	VtblCall__IJ_3J_FUNC,
	VtblCall_1put_1Bounds_FUNC,
	WriteClassStg_FUNC,
} COM_FUNCS;
