/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

#ifndef SWT_GLX_H_INCLUDED
#define SWT_GLX_H_INCLUDED

#include <GL/glx.h>
#include <X11/Xutil.h>
#include <string.h>

#endif /* SWT_GLX_H_INCLUDED */
