/*******************************************************************************
 * Copyright (c) 2000, 2023 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *    IBM Corporation - initial API and implementation
 *******************************************************************************/

/* Note: This file was auto-generated by org.eclipse.swt.tools.internal.JNIGenerator */
/* DO NOT EDIT - your changes will be lost. */

#ifndef OS_NATIVE_ENTER
#define OS_NATIVE_ENTER(env, that, func) 
#endif
#ifndef OS_NATIVE_EXIT
#define OS_NATIVE_EXIT(env, that, func) 
#endif

typedef enum {
	ACCEL_1sizeof_FUNC,
	ACTCTX_1sizeof_FUNC,
	AbortDoc_FUNC,
	ActivateActCtx_FUNC,
	ActivateKeyboardLayout_FUNC,
	AddFontResourceEx_FUNC,
	AdjustWindowRectEx_FUNC,
	AllowDarkModeForWindow_FUNC,
	AllowSetForegroundWindow_FUNC,
	AlphaBlend_FUNC,
	Arc_FUNC,
	AssocQueryString_FUNC,
	BITMAPINFOHEADER_1sizeof_FUNC,
	BITMAP_1sizeof_FUNC,
	BLENDFUNCTION_1sizeof_FUNC,
	BP_1PAINTPARAMS_1sizeof_FUNC,
	BUTTON_1IMAGELIST_1sizeof_FUNC,
	BeginBufferedPaint_FUNC,
	BeginDeferWindowPos_FUNC,
	BeginPaint_FUNC,
	BitBlt_FUNC,
	BringWindowToTop_FUNC,
	BufferedPaintInit_FUNC,
	BufferedPaintUnInit_FUNC,
	CANDIDATEFORM_1sizeof_FUNC,
	CHOOSECOLOR_1sizeof_FUNC,
	CHOOSEFONT_1sizeof_FUNC,
	CIDA_1sizeof_FUNC,
	COMBOBOXINFO_1sizeof_FUNC,
	COMPOSITIONFORM_1sizeof_FUNC,
	CREATESTRUCT_1sizeof_FUNC,
	CallNextHookEx_FUNC,
	CallWindowProc_FUNC,
	CharLower_FUNC,
	CharUpper_FUNC,
	ChildWindowFromPointEx_FUNC,
	ChooseColor_FUNC,
	ChooseFont_FUNC,
	ClientToScreen_FUNC,
	CloseClipboard_FUNC,
	CloseEnhMetaFile_FUNC,
	CloseGestureInfoHandle_FUNC,
	CloseHandle_FUNC,
	ClosePrinter_FUNC,
	CloseThemeData_FUNC,
	CloseTouchInputHandle_FUNC,
	CoInternetIsFeatureEnabled_FUNC,
	CoInternetSetFeatureEnabled_FUNC,
	CoTaskMemAlloc_FUNC,
	CoTaskMemFree_FUNC,
	CombineRgn_FUNC,
	CopyImage_FUNC,
	CreateAcceleratorTable_FUNC,
	CreateActCtx_FUNC,
	CreateBitmap_FUNC,
	CreateCaret_FUNC,
	CreateCompatibleBitmap_FUNC,
	CreateCompatibleDC_FUNC,
	CreateCursor_FUNC,
	CreateDC_FUNC,
	CreateDIBSection__JJI_3JJI_FUNC,
	CreateDIBSection__J_3BI_3JJI_FUNC,
	CreateEnhMetaFile_FUNC,
	CreateFontIndirect__J_FUNC,
	CreateFontIndirect__Lorg_eclipse_swt_internal_win32_LOGFONT_2_FUNC,
	CreateIconIndirect_FUNC,
	CreateMenu_FUNC,
	CreatePatternBrush_FUNC,
	CreatePen_FUNC,
	CreatePolygonRgn_FUNC,
	CreatePopupMenu_FUNC,
	CreateProcess_FUNC,
	CreateRectRgn_FUNC,
	CreateSolidBrush_FUNC,
	CreateStreamOnHGlobal_FUNC,
	CreateWindowEx_FUNC,
	DEVMODE_1sizeof_FUNC,
	DIBSECTION_1sizeof_FUNC,
	DOCHOSTUIINFO_1sizeof_FUNC,
	DOCINFO_1sizeof_FUNC,
	DRAWITEMSTRUCT_1sizeof_FUNC,
	DROPFILES_1sizeof_FUNC,
	DefFrameProc_FUNC,
	DefMDIChildProc_FUNC,
	DefWindowProc_FUNC,
	DeferWindowPos_FUNC,
	DeleteDC_FUNC,
	DeleteEnhMetaFile_FUNC,
	DeleteMenu_FUNC,
	DeleteObject_FUNC,
	DestroyAcceleratorTable_FUNC,
	DestroyCaret_FUNC,
	DestroyCursor_FUNC,
	DestroyIcon_FUNC,
	DestroyMenu_FUNC,
	DestroyWindow_FUNC,
	DispatchMessage_FUNC,
	DocumentProperties_FUNC,
	DragDetect_FUNC,
	DragFinish_FUNC,
	DragQueryFile_FUNC,
	DrawEdge_FUNC,
	DrawFocusRect_FUNC,
	DrawFrameControl_FUNC,
	DrawIconEx_FUNC,
	DrawMenuBar_FUNC,
	DrawText_FUNC,
	DrawThemeBackground_FUNC,
	DrawThemeText_FUNC,
	DuplicateHandle_FUNC,
	DwmSetWindowAttribute_FUNC,
	EMREXTCREATEFONTINDIRECTW_1sizeof_FUNC,
	EMR_1sizeof_FUNC,
	EXTLOGFONTW_1sizeof_FUNC,
	Ellipse_FUNC,
	EnableMenuItem_FUNC,
	EnableScrollBar_FUNC,
	EnableWindow_FUNC,
	EndBufferedPaint_FUNC,
	EndDeferWindowPos_FUNC,
	EndDoc_FUNC,
	EndPage_FUNC,
	EndPaint_FUNC,
	EnumDisplayMonitors_FUNC,
	EnumEnhMetaFile_FUNC,
	EnumFontFamilies_FUNC,
	EnumSystemLanguageGroups_FUNC,
	EnumSystemLocales_FUNC,
	EqualRect_FUNC,
	ExcludeClipRect_FUNC,
	ExpandEnvironmentStrings_FUNC,
	ExtCreatePen_FUNC,
	ExtCreateRegion_FUNC,
	ExtTextOut_FUNC,
	ExtractIconEx_FUNC,
	FLICK_1DATA_1sizeof_FUNC,
	FLICK_1POINT_1sizeof_FUNC,
	FillRect_FUNC,
	GCP_1RESULTS_1sizeof_FUNC,
	GESTURECONFIG_1sizeof_FUNC,
	GESTUREINFO_1sizeof_FUNC,
	GID_1ROTATE_1ANGLE_1FROM_1ARGUMENT_FUNC,
	GRADIENT_1RECT_1sizeof_FUNC,
	GUITHREADINFO_1sizeof_FUNC,
	GdiSetBatchLimit_FUNC,
	GetACP_FUNC,
	GetActiveWindow_FUNC,
	GetBkColor_FUNC,
	GetCapture_FUNC,
	GetCaretPos_FUNC,
	GetCharABCWidths_FUNC,
	GetCharWidth_FUNC,
	GetCharacterPlacement_FUNC,
	GetClassInfo_FUNC,
	GetClassName_FUNC,
	GetClientRect_FUNC,
	GetClipBox_FUNC,
	GetClipRgn_FUNC,
	GetClipboardData_FUNC,
	GetClipboardFormatName_FUNC,
	GetComboBoxInfo_FUNC,
	GetCurrentObject_FUNC,
	GetCurrentProcess_FUNC,
	GetCurrentProcessExplicitAppUserModelID_FUNC,
	GetCurrentProcessId_FUNC,
	GetCurrentThreadId_FUNC,
	GetCursor_FUNC,
	GetCursorPos_FUNC,
	GetDC_FUNC,
	GetDCEx_FUNC,
	GetDIBColorTable_FUNC,
	GetDIBits_FUNC,
	GetDesktopWindow_FUNC,
	GetDeviceCaps_FUNC,
	GetDialogBaseUnits_FUNC,
	GetDlgItem_FUNC,
	GetDoubleClickTime_FUNC,
	GetDpiForMonitor_FUNC,
	GetFocus_FUNC,
	GetFontLanguageInfo_FUNC,
	GetForegroundWindow_FUNC,
	GetGUIThreadInfo_FUNC,
	GetGestureInfo_FUNC,
	GetGlyphIndices_FUNC,
	GetGraphicsMode_FUNC,
	GetIconInfo_FUNC,
	GetKeyState_FUNC,
	GetKeyboardLayout_FUNC,
	GetKeyboardLayoutList_FUNC,
	GetKeyboardState_FUNC,
	GetLastActivePopup_FUNC,
	GetLastError_FUNC,
	GetLayeredWindowAttributes_FUNC,
	GetLayout_FUNC,
	GetLibraryHandle_FUNC,
	GetLocaleInfo_FUNC,
	GetMenu_FUNC,
	GetMenuBarInfo_FUNC,
	GetMenuDefaultItem_FUNC,
	GetMenuInfo_FUNC,
	GetMenuItemCount_FUNC,
	GetMenuItemInfo_FUNC,
	GetMenuItemRect_FUNC,
	GetMessage_FUNC,
	GetMessagePos_FUNC,
	GetMessageTime_FUNC,
	GetMetaRgn_FUNC,
	GetModuleFileName_FUNC,
	GetModuleHandle_FUNC,
	GetMonitorInfo_FUNC,
	GetObject__JIJ_FUNC,
	GetObject__JILorg_eclipse_swt_internal_win32_BITMAP_2_FUNC,
	GetObject__JILorg_eclipse_swt_internal_win32_DIBSECTION_2_FUNC,
	GetObject__JILorg_eclipse_swt_internal_win32_LOGBRUSH_2_FUNC,
	GetObject__JILorg_eclipse_swt_internal_win32_LOGFONT_2_FUNC,
	GetOutlineTextMetrics_FUNC,
	GetParent_FUNC,
	GetPixel_FUNC,
	GetPolyFillMode_FUNC,
	GetProcessHeap_FUNC,
	GetProfileString_FUNC,
	GetProp_FUNC,
	GetROP2_FUNC,
	GetRandomRgn_FUNC,
	GetRegionData_FUNC,
	GetRgnBox_FUNC,
	GetScrollBarInfo_FUNC,
	GetScrollInfo_FUNC,
	GetStartupInfo_FUNC,
	GetStockObject_FUNC,
	GetSysColor_FUNC,
	GetSysColorBrush_FUNC,
	GetSystemMenu_FUNC,
	GetSystemMetrics_FUNC,
	GetTextColor_FUNC,
	GetTextExtentPoint32_FUNC,
	GetTextMetrics_FUNC,
	GetThemePartSize_FUNC,
	GetThemeTextExtent_FUNC,
	GetTouchInputInfo_FUNC,
	GetUpdateRect_FUNC,
	GetUpdateRgn_FUNC,
	GetWindow_FUNC,
	GetWindowDC_FUNC,
	GetWindowLong_FUNC,
	GetWindowLongPtr_FUNC,
	GetWindowOrgEx_FUNC,
	GetWindowPlacement_FUNC,
	GetWindowRect_FUNC,
	GetWindowRgn_FUNC,
	GetWindowText_FUNC,
	GetWindowTextLength_FUNC,
	GetWindowThreadProcessId_FUNC,
	GlobalAddAtom_FUNC,
	GlobalAlloc_FUNC,
	GlobalFree_FUNC,
	GlobalLock_FUNC,
	GlobalSize_FUNC,
	GlobalUnlock_FUNC,
	GradientFill_FUNC,
	HDHITTESTINFO_1sizeof_FUNC,
	HDITEM_1sizeof_FUNC,
	HDLAYOUT_1sizeof_FUNC,
	HELPINFO_1sizeof_FUNC,
	HIGHCONTRAST_1sizeof_FUNC,
	HeapAlloc_FUNC,
	HeapFree_FUNC,
	HideCaret_FUNC,
	ICONINFO_1sizeof_FUNC,
	IIDFromString_FUNC,
	ILGetSize_FUNC,
	INITCOMMONCONTROLSEX_1sizeof_FUNC,
	INPUT_1sizeof_FUNC,
	ImageList_1Add_FUNC,
	ImageList_1AddMasked_FUNC,
	ImageList_1BeginDrag_FUNC,
	ImageList_1Create_FUNC,
	ImageList_1Destroy_FUNC,
	ImageList_1DragEnter_FUNC,
	ImageList_1DragLeave_FUNC,
	ImageList_1DragMove_FUNC,
	ImageList_1DragShowNolock_FUNC,
	ImageList_1EndDrag_FUNC,
	ImageList_1GetIconSize_FUNC,
	ImageList_1GetImageCount_FUNC,
	ImageList_1Remove_FUNC,
	ImageList_1Replace_FUNC,
	ImageList_1ReplaceIcon_FUNC,
	ImageList_1SetIconSize_FUNC,
	ImmEscape_FUNC,
	ImmGetCompositionFont_FUNC,
	ImmGetCompositionString__JI_3BI_FUNC,
	ImmGetCompositionString__JI_3CI_FUNC,
	ImmGetCompositionString__JI_3II_FUNC,
	ImmGetContext_FUNC,
	ImmGetConversionStatus_FUNC,
	ImmGetDefaultIMEWnd_FUNC,
	ImmGetOpenStatus_FUNC,
	ImmNotifyIME_FUNC,
	ImmReleaseContext_FUNC,
	ImmSetCandidateWindow_FUNC,
	ImmSetCompositionFont_FUNC,
	ImmSetCompositionWindow_FUNC,
	ImmSetConversionStatus_FUNC,
	ImmSetOpenStatus_FUNC,
	InitCommonControlsEx_FUNC,
	InsertMenuItem_FUNC,
	InternetGetCookie_FUNC,
	InternetSetCookie_FUNC,
	InternetSetOption_FUNC,
	IntersectClipRect_FUNC,
	IntersectRect_FUNC,
	InvalidateRect_FUNC,
	InvalidateRgn_FUNC,
	IsAppThemed_FUNC,
	IsDarkModeAvailable_FUNC,
	IsHungAppWindow_FUNC,
	IsIconic_FUNC,
	IsTouchWindow_FUNC,
	IsWindowEnabled_FUNC,
	IsWindowVisible_FUNC,
	IsZoomed_FUNC,
	KEYBDINPUT_1sizeof_FUNC,
	KillTimer_FUNC,
	LITEM_1sizeof_FUNC,
	LOGBRUSH_1sizeof_FUNC,
	LOGFONT_1sizeof_FUNC,
	LOGPEN_1sizeof_FUNC,
	LPtoDP_FUNC,
	LVCOLUMN_1sizeof_FUNC,
	LVHITTESTINFO_1sizeof_FUNC,
	LVINSERTMARK_1sizeof_FUNC,
	LVITEM_1sizeof_FUNC,
	LineTo_FUNC,
	LoadBitmap_FUNC,
	LoadCursor_FUNC,
	LoadIcon_FUNC,
	LoadIconMetric_FUNC,
	LoadImage_FUNC,
	LoadKeyboardLayout_FUNC,
	LocalFree_FUNC,
	MARGINS_1sizeof_FUNC,
	MCHITTESTINFO_1sizeof_FUNC,
	MEASUREITEMSTRUCT_1sizeof_FUNC,
	MENUBARINFO_1sizeof_FUNC,
	MENUINFO_1sizeof_FUNC,
	MENUITEMINFO_1sizeof_FUNC,
	MINMAXINFO_1sizeof_FUNC,
	MONITORINFO_1sizeof_FUNC,
	MOUSEINPUT_1sizeof_FUNC,
	MSG_1sizeof_FUNC,
	MapViewOfFile_FUNC,
	MapVirtualKey_FUNC,
	MapWindowPoints__JJLorg_eclipse_swt_internal_win32_POINT_2I_FUNC,
	MapWindowPoints__JJLorg_eclipse_swt_internal_win32_RECT_2I_FUNC,
	MessageBeep_FUNC,
	MessageBox_FUNC,
	ModifyWorldTransform_FUNC,
	MonitorFromWindow_FUNC,
	MoveMemory__JJI_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_CIDA_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_DEVMODE_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_DOCHOSTUIINFO_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_DROPFILES_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_GRADIENT_1RECT_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_LOGFONT_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_MEASUREITEMSTRUCT_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_MINMAXINFO_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_MSG_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_NMLVCUSTOMDRAW_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_NMLVDISPINFO_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_NMTBCUSTOMDRAW_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_NMTTDISPINFO_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_NMTVCUSTOMDRAW_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_NMTVDISPINFO_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_RECT_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_SAFEARRAY_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_SHDRAGIMAGE_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_TRIVERTEX_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_UDACCEL_2I_FUNC,
	MoveMemory__JLorg_eclipse_swt_internal_win32_WINDOWPOS_2I_FUNC,
	MoveMemory__J_3BI_FUNC,
	MoveMemory__J_3CI_FUNC,
	MoveMemory__J_3DI_FUNC,
	MoveMemory__J_3FI_FUNC,
	MoveMemory__J_3II_FUNC,
	MoveMemory__J_3JI_FUNC,
	MoveMemory__J_3SI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_BITMAPINFOHEADER_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_BITMAPINFOHEADER_2_3BI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_CIDA_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_DEVMODE_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_DOCHOSTUIINFO_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_DRAWITEMSTRUCT_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_EMREXTCREATEFONTINDIRECTW_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_EMR_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_FLICK_1DATA_2_3JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_FLICK_1POINT_2_3JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_HDITEM_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_HELPINFO_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_LOGFONT_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_MEASUREITEMSTRUCT_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_MINMAXINFO_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_MSG_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMCUSTOMDRAW_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMHDR_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMHEADER_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMLINK_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMLISTVIEW_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMLVCUSTOMDRAW_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMLVDISPINFO_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMLVODSTATECHANGE_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMREBARCHEVRON_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMREBARCHILDSIZE_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMTBCUSTOMDRAW_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMTBHOTITEM_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMTOOLBAR_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMTREEVIEW_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMTTCUSTOMDRAW_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMTTDISPINFO_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMTVCUSTOMDRAW_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMTVDISPINFO_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMTVITEMCHANGE_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_NMUPDOWN_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_POINT_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_POINT_2_3JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_SAFEARRAY_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_SCRIPT_1ITEM_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_SCRIPT_1LOGATTR_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_SCRIPT_1PROPERTIES_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_TEXTMETRIC_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_TOUCHINPUT_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_UDACCEL_2JI_FUNC,
	MoveMemory__Lorg_eclipse_swt_internal_win32_WINDOWPOS_2JI_FUNC,
	MoveMemory___3BJI_FUNC,
	MoveMemory___3BLorg_eclipse_swt_internal_win32_ACCEL_2I_FUNC,
	MoveMemory___3BLorg_eclipse_swt_internal_win32_BITMAPINFOHEADER_2I_FUNC,
	MoveMemory___3CJI_FUNC,
	MoveMemory___3DJI_FUNC,
	MoveMemory___3FJI_FUNC,
	MoveMemory___3IJI_FUNC,
	MoveMemory___3JJI_FUNC,
	MoveMemory___3SJI_FUNC,
	MoveToEx_FUNC,
	MultiByteToWideChar__IIJI_3CI_FUNC,
	MultiByteToWideChar__II_3BI_3CI_FUNC,
	NMCUSTOMDRAW_1sizeof_FUNC,
	NMHDR_1sizeof_FUNC,
	NMHEADER_1sizeof_FUNC,
	NMLINK_1sizeof_FUNC,
	NMLISTVIEW_1sizeof_FUNC,
	NMLVCUSTOMDRAW_1sizeof_FUNC,
	NMLVDISPINFO_1sizeof_FUNC,
	NMLVFINDITEM_1sizeof_FUNC,
	NMLVODSTATECHANGE_1sizeof_FUNC,
	NMREBARCHEVRON_1sizeof_FUNC,
	NMREBARCHILDSIZE_1sizeof_FUNC,
	NMTBCUSTOMDRAW_1sizeof_FUNC,
	NMTBHOTITEM_1sizeof_FUNC,
	NMTOOLBAR_1sizeof_FUNC,
	NMTREEVIEW_1sizeof_FUNC,
	NMTTCUSTOMDRAW_1sizeof_FUNC,
	NMTTDISPINFO_1sizeof_FUNC,
	NMTVCUSTOMDRAW_1sizeof_FUNC,
	NMTVDISPINFO_1sizeof_FUNC,
	NMTVITEMCHANGE_1sizeof_FUNC,
	NMUPDOWN_1sizeof_FUNC,
	NONCLIENTMETRICS_1sizeof_FUNC,
	NOTIFYICONDATA_1V2_1SIZE_FUNC,
	NotifyWinEvent_FUNC,
	OSVERSIONINFOEX_1sizeof_FUNC,
	OUTLINETEXTMETRIC_1sizeof_FUNC,
	OffsetRect_FUNC,
	OffsetRgn_FUNC,
	OleInitialize_FUNC,
	OleUninitialize_FUNC,
	OpenClipboard_FUNC,
	OpenPrinter_FUNC,
	OpenProcess_FUNC,
	OpenThemeData_FUNC,
	PAINTSTRUCT_1sizeof_FUNC,
	POINT_1sizeof_FUNC,
	PRINTDLG_1sizeof_FUNC,
	PROCESS_1INFORMATION_1sizeof_FUNC,
	PROPERTYKEY_1sizeof_FUNC,
	PROPVARIANT_1sizeof_FUNC,
	PSPropertyKeyFromString_FUNC,
	PatBlt_FUNC,
	PathIsExe_FUNC,
	PeekMessage_FUNC,
	Pie_FUNC,
	Polygon_FUNC,
	Polyline_FUNC,
	PostMessage_FUNC,
	PostThreadMessage_FUNC,
	PrintDlg_FUNC,
	PrintWindow_FUNC,
	PtInRect_FUNC,
	PtInRegion_FUNC,
	REBARBANDINFO_1sizeof_FUNC,
	RECT_1sizeof_FUNC,
	RectInRegion_FUNC,
	Rectangle_FUNC,
	RedrawWindow_FUNC,
	RegCloseKey_FUNC,
	RegCreateKeyEx_FUNC,
	RegDeleteValue_FUNC,
	RegEnumKeyEx_FUNC,
	RegOpenKeyEx_FUNC,
	RegQueryValueEx__J_3CJ_3I_3C_3I_FUNC,
	RegQueryValueEx__J_3CJ_3I_3I_3I_FUNC,
	RegSetValueEx_FUNC,
	RegisterClass_FUNC,
	RegisterClipboardFormat_FUNC,
	RegisterHotKey_FUNC,
	RegisterTouchWindow_FUNC,
	RegisterWindowMessage_FUNC,
	ReleaseCapture_FUNC,
	ReleaseDC_FUNC,
	RemoveMenu_FUNC,
	RemoveProp_FUNC,
	ReplyMessage_FUNC,
	RestoreDC_FUNC,
	RoundRect_FUNC,
	RtlGetVersion_FUNC,
	SAFEARRAYBOUND_1sizeof_FUNC,
	SAFEARRAY_1sizeof_FUNC,
	SCRIPT_1ANALYSIS_1sizeof_FUNC,
	SCRIPT_1CONTROL_1sizeof_FUNC,
	SCRIPT_1FONTPROPERTIES_1sizeof_FUNC,
	SCRIPT_1ITEM_1sizeof_FUNC,
	SCRIPT_1LOGATTR_1sizeof_FUNC,
	SCRIPT_1PROPERTIES_1sizeof_FUNC,
	SCRIPT_1STATE_1sizeof_FUNC,
	SCRIPT_1STRING_1ANALYSIS_1sizeof_FUNC,
	SCROLLBARINFO_1sizeof_FUNC,
	SCROLLINFO_1sizeof_FUNC,
	SHDRAGIMAGE_1sizeof_FUNC,
	SHELLEXECUTEINFO_1sizeof_FUNC,
	SHFILEINFO_1sizeof_FUNC,
	SHGetFileInfo_FUNC,
	SIZE_1sizeof_FUNC,
	STARTUPINFO_1sizeof_FUNC,
	SYSTEMTIME_1sizeof_FUNC,
	SaveDC_FUNC,
	ScreenToClient_FUNC,
	ScriptApplyDigitSubstitution_FUNC,
	ScriptBreak_FUNC,
	ScriptCPtoX_FUNC,
	ScriptCacheGetHeight_FUNC,
	ScriptFreeCache_FUNC,
	ScriptGetCMap_FUNC,
	ScriptGetFontProperties_FUNC,
	ScriptGetLogicalWidths_FUNC,
	ScriptGetProperties_FUNC,
	ScriptItemize_FUNC,
	ScriptJustify_FUNC,
	ScriptLayout_FUNC,
	ScriptPlace_FUNC,
	ScriptShape_FUNC,
	ScriptStringAnalyse_FUNC,
	ScriptStringFree_FUNC,
	ScriptStringOut_FUNC,
	ScriptTextOut_FUNC,
	ScriptXtoCP_FUNC,
	ScrollWindowEx_FUNC,
	SelectClipRgn_FUNC,
	SelectObject_FUNC,
	SendInput_FUNC,
	SendMessage__JIJJ_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_BUTTON_1IMAGELIST_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_HDHITTESTINFO_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_HDITEM_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_HDLAYOUT_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_LITEM_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_LVCOLUMN_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_LVHITTESTINFO_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_LVINSERTMARK_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_LVITEM_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_MARGINS_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_MCHITTESTINFO_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_REBARBANDINFO_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_RECT_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_SHDRAGIMAGE_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_SIZE_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_SYSTEMTIME_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_TBBUTTONINFO_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_TBBUTTON_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_TCHITTESTINFO_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_TCITEM_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_TOOLINFO_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_TVHITTESTINFO_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_TVINSERTSTRUCT_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_TVITEM_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_TVSORTCB_2_FUNC,
	SendMessage__JIJLorg_eclipse_swt_internal_win32_UDACCEL_2_FUNC,
	SendMessage__JIJ_3C_FUNC,
	SendMessage__JIJ_3I_FUNC,
	SendMessage__JI_3I_3I_FUNC,
	SetActiveWindow_FUNC,
	SetBkColor_FUNC,
	SetBkMode_FUNC,
	SetBrushOrgEx_FUNC,
	SetCapture_FUNC,
	SetCaretPos_FUNC,
	SetCurrentProcessExplicitAppUserModelID_FUNC,
	SetCursor_FUNC,
	SetCursorPos_FUNC,
	SetDCBrushColor_FUNC,
	SetDIBColorTable_FUNC,
	SetFocus_FUNC,
	SetForegroundWindow_FUNC,
	SetGestureConfig_FUNC,
	SetGraphicsMode_FUNC,
	SetLayeredWindowAttributes_FUNC,
	SetLayout_FUNC,
	SetMenu_FUNC,
	SetMenuDefaultItem_FUNC,
	SetMenuInfo_FUNC,
	SetMenuItemInfo_FUNC,
	SetMetaRgn_FUNC,
	SetParent_FUNC,
	SetPixel_FUNC,
	SetPolyFillMode_FUNC,
	SetPreferredAppMode_FUNC,
	SetProcessDPIAware_FUNC,
	SetProp_FUNC,
	SetROP2_FUNC,
	SetRect_FUNC,
	SetRectRgn_FUNC,
	SetScrollInfo_FUNC,
	SetStretchBltMode_FUNC,
	SetTextColor_FUNC,
	SetTimer_FUNC,
	SetWindowLong_FUNC,
	SetWindowLongPtr_FUNC,
	SetWindowOrgEx_FUNC,
	SetWindowPlacement_FUNC,
	SetWindowPos_FUNC,
	SetWindowRgn_FUNC,
	SetWindowText_FUNC,
	SetWindowTheme_FUNC,
	SetWindowsHookEx_FUNC,
	SetWorldTransform_FUNC,
	ShellExecuteEx_FUNC,
	Shell_1NotifyIcon_FUNC,
	ShowCaret_FUNC,
	ShowOwnedPopups_FUNC,
	ShowScrollBar_FUNC,
	ShowWindow_FUNC,
	StartDoc_FUNC,
	StartPage_FUNC,
	StretchBlt_FUNC,
	SystemParametersInfo__IILorg_eclipse_swt_internal_win32_HIGHCONTRAST_2I_FUNC,
	SystemParametersInfo__IILorg_eclipse_swt_internal_win32_NONCLIENTMETRICS_2I_FUNC,
	SystemParametersInfo__IILorg_eclipse_swt_internal_win32_RECT_2I_FUNC,
	SystemParametersInfo__II_3II_FUNC,
	TBBUTTONINFO_1sizeof_FUNC,
	TBBUTTON_1sizeof_FUNC,
	TCHITTESTINFO_1sizeof_FUNC,
	TCITEM_1sizeof_FUNC,
	TEXTMETRIC_1sizeof_FUNC,
	TF_1DA_1COLOR_1sizeof_FUNC,
	TF_1DISPLAYATTRIBUTE_1sizeof_FUNC,
	TOOLINFO_1sizeof_FUNC,
	TOUCHINPUT_1sizeof_FUNC,
	TRACKMOUSEEVENT_1sizeof_FUNC,
	TRIVERTEX_1sizeof_FUNC,
	TVHITTESTINFO_1sizeof_FUNC,
	TVINSERTSTRUCT_1sizeof_FUNC,
	TVITEM_1sizeof_FUNC,
	TVSORTCB_1sizeof_FUNC,
	ToUnicode_FUNC,
	TrackMouseEvent_FUNC,
	TrackPopupMenu_FUNC,
	TranslateAccelerator_FUNC,
	TranslateCharsetInfo_FUNC,
	TranslateMDISysAccel_FUNC,
	TranslateMessage_FUNC,
	TransparentBlt_FUNC,
	TreeView_1GetItemRect_FUNC,
	UDACCEL_1sizeof_FUNC,
	UnhookWindowsHookEx_FUNC,
	UnloadKeyboardLayout_FUNC,
	UnmapViewOfFile_FUNC,
	UnregisterClass_FUNC,
	UnregisterHotKey_FUNC,
	UnregisterTouchWindow_FUNC,
	UpdateWindow_FUNC,
	UrlCreateFromPath_FUNC,
	ValidateRect_FUNC,
	VkKeyScan_FUNC,
	WINDOWPLACEMENT_1sizeof_FUNC,
	WINDOWPOS_1sizeof_FUNC,
	WNDCLASS_1sizeof_FUNC,
	WaitMessage_FUNC,
	WideCharToMultiByte__II_3CIJI_3B_3I_FUNC,
	WideCharToMultiByte__II_3CI_3BI_3B_3I_FUNC,
	WindowFromDC_FUNC,
	WindowFromPoint_FUNC,
	wcslen_FUNC,
} OS_FUNCS;
