/*******************************************************************************
 * Copyright (c) 2000, 2015 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.swt.tests.junit;


import org.eclipse.test.TracingSuite;
import org.eclipse.test.TracingSuite.TracingOptions;
import org.junit.runner.JUnitCore;
import org.junit.runner.RunWith;
import org.junit.runners.Suite;

@RunWith(TracingSuite.class)
@TracingOptions(stackDumpTimeoutSeconds = 60)
@Suite.SuiteClasses({
	Test_org_eclipse_swt_browser_Browser.class,
})
public class AllBrowserTests {

public static void main(String[] args) {
	JUnitCore.main(AllBrowserTests.class.getName());
}
}
